<?php

namespace Modules\Konfirmasibank\Entities;

use Spatie\Activitylog\LogOptions;
use Spatie\Activitylog\Traits\LogsActivity;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Carbon\Carbon;

class Account extends Model
{
    use HasFactory;

    protected $connection = 'db2';
    protected $table = 'STG_DB.ACCOUNT';
    protected $primaryKey = '@ID';
    protected $fillable = [
    "'CUSTOMER_NO',
    'MNEMONIC'
    'CATEGORY'"
    ];



    public function arrangement()
    {
        return $this->belongsTo(Arrangement::class, 'LINKED_APPL_ID');
    }

    public function getSearchAccount($cif, $arrAccount, $kodecabang, $startDate)
    {
       // dd($startDate);

        $uppercaseKodeCabang = Str::upper($kodecabang);
        $data = [];


        if($arrAccount == null) {
            $data = DB::connection("db2")->table("STG_DB.ACCOUNT AS AC")
             ->selectRaw('AC.CUSTOMER_NO,AC.ACCOUNT_NUMBER,AC.BATCH_DATE,AC.WORKING_BALANCE,CP.COMPANY_NAME,AC.CURRENCY,CT.SHORT_NAME')
             ->join('STG_DB.CATEGORY AS CT', 'AC.CATEGORY', '=', 'CT.ID')
            ->join('STG_DB.COMPANY AS CP', 'AC.CO_CODE', '=', 'CP.ID')
            ->where('CUSTOMER_NO', $cif)
            ->where('COMPANY_NAME', 'like', "%" .$uppercaseKodeCabang."%")
           // ->where('ARR_A', null)
            ->where('AC.BATCH_DATE', $startDate)
           // ->whereIn(DB::raw("LEFT(CATEGORY, 3)"), ['100','600','660','101'])
            ->whereIn(DB::raw("LEFT(CATEGORY, 2)"), ['10','60','66','30','31','32','33'])
           // ->orWhere(DB::raw("LEFT(CATEGORY, 2)"),'32')
            ->groupBy('CUSTOMER_NO','ACCOUNT_NUMBER','AC.BATCH_DATE','WORKING_BALANCE','COMPANY_NAME','CURRENCY','SHORT_NAME')
            ->limit(100);
        } else {
            $data = DB::connection("db2")->table("STG_DB.ACCOUNT")
            ->select('*')
            ->where('CUSTOMER_NO', $cif)
            ->where('COMPANY_NAME', 'like', "%" .$uppercaseKodeCabang."%")
            ->whereIn('ACCOUNT_NUMBER', $arrAccount)
            ->where('OPENING_DATE', $startDate);
        }

        return $data;
    }

    public function getCekProduct($cust ,$acc, $startDate){
       // dd($cust, $arrAccount, $startDate);
        $timestamp = strtotime($startDate);
        // Format the timestamp into a date format
        $Year = date("Y", $timestamp);
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.AA_ARRANGEMENT")->select('*')
            ->where('CUSTOMER', $cust)
            ->where('LINKED_APPL_ID', $acc)
          //  ->whereYear('BATCH_DATE', $Year)
            ->where('ARR_STATUS','EXPIRED')
            ->limit(1);
        return $data;

    }

          public function getAccount($cif, $arrAccount, $startDate)
    {
        //$uppercaseKodeCabang = Str::upper($kodecabang);
          // $dateString = $startDate->format('m/d/Y');

           $data = DB::connection("db2")->table("STG_DB.ACCOUNT AS AC")
         ->selectRaw('AC.CUSTOMER_NO,AC.CATEGORY,AC.ACCOUNT_NUMBER,AC.CURRENCY, AC.SHORT_TITLE,AC.OPENING_DATE,AC.WORKING_BALANCE, AC.CATEGORY,CT.SHORT_NAME,CP.COMPANY_NAME,AC.BATCH_DATE')
          ->join('STG_DB.CATEGORY AS CT', 'AC.CATEGORY', '=', 'CT.ID')
          ->join('STG_DB.COMPANY AS CP', 'AC.CO_CODE', '=', 'CP.ID')
          ->where('CUSTOMER_NO', $cif)
          ->where('LIMIT_REF', null)
          ->where('AC.BATCH_DATE', $startDate)
          ->whereIn('AC.ACCOUNT_NUMBER', $arrAccount)
          ->whereIn(DB::raw("LEFT(CATEGORY, 3)"),['100','101','600'])
         // ->groupBy('CUSTOMER_NO', 'ACCOUNT_NUMBER', 'SHORT_TITLE', 'SHORT_NAME', 'COMPANY_NAME', 'CATEGORY', 'LIMIT_REF')
          ->orderBy('CATEGORY', 'ASC')
          ->limit(100);
        return $data;
    }



    public function getAA($cust, $acc, $dataCore)
    {
     //   dd($cust, $acc, $dataCore);
        //$uppercaseKodeCabang = Str::upper($kodecabang);
       $dateString = $dataCore->format('m/d/Y');
       // dd($dateString);
        $timestamp = strtotime($dataCore);
        // Format the timestamp into a date format
        $Year = date("Y", $timestamp);

        $data = DB::connection("db2")->table("STG_DB.AA_ARRANGEMENT AS ARR")
           ->selectRaw('ARR.ID,ARR.LINKED_APPL_ID,CUSTOMER,ARR.PRODUCT_LINE,AD.MATURITY_DATE,AI.FIXED_RATE,ARR.BATCH_DATE')
          ->join('STG_DB.AA_ACCOUNT_DETAILS AS AD', 'ARR.ID', '=', 'AD.ID')
          ->join('STG_DB.AA_ARR_INTEREST AS AI', DB::raw("LEFT(ARR.ID,12)"), '=', 'AD.ID')
          ->where('ARR.CUSTOMER', $cust)
          ->where('ARR.BATCH_DATE',  $dateString)
          ->where('ARR.LINKED_APPL_ID', $acc)
          ->groupBy('ARR.ID', 'ARR.LINKED_APPL_ID', 'ARR.CUSTOMER', 'ARR.PRODUCT_LINE','ARR.BATCH_DATE','AD.MATURITY_DATE','AI.FIXED_RATE','ARR.BATCH_DATE')
        //  ->orderBy('ARR.BATCH_DATE', 'DESC')
          ->orderBy('ARR.BATCH_DATE', 'DESC')
          ->limit(1);
        return $data;
    }


    public function getDepo($cust, $acc, $dataCore){
        $timestamp = strtotime($dataCore);
        $date = date("m/d/Y", $timestamp);
        $data = [];
        $data = DB::connection("db2")->table("MIS.VW_AA_DEPO")->select('*')
            ->whereIn('NO_REK', $acc)
            ->where('PERIOD', $date)
            ->limit(100);
        return $data;
    }

    public function getCustomer($cus_no)
    {
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.CUSTOMER")->select('CUSTOMER_NO', 'STREET', 'RESIDENCE', 'ADDRESS', 'SHORT_NAME')->where('CUSTOMER_NO', $cus_no);
        return $data;
    }


    public function getAAaccount($cust, $acc)
    {
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.VW_AA_ACCOUNT")->select('ARRANGEMENT_ID', 'MATURITY_DATE', 'PRODUCT', 'TERM', 'RENEWAL_DATE', 'START_DATE', 'FIXED_RATE')
            ->where('CUSTOMER', $cust)
            ->where('LINKED_APPL_ID', $acc)
            ->limit(1);
        return $data;
    }


    public function getPinjaman($cus_no, $dateCore)
    {

        $data = [];
        $data = DB::connection("db2")->table("STG_DB.VW_AA_ARR")
        ->select('LINKED_APPL_ID', 'ARRANGEMENT_ID', 'CUSTOMER', 'BATCH_DATE', 'START_DATE', 'PRODUCT_LINE', 'PRODUCT', 'ARR_STATUS', 'CURRENCY', 'TERM', 'MATURITY_DATE', 'RENEWAL_DATE', 'FIXED_RATE')
        ->where('CUSTOMER', $cus_no)
        ->where('ARR_STATUS', 'CURRENT')
        ->where('BATCH_DATE', $dateCore)
        ->where('ARR_STATUS', '<>', 'CLOSE')
        ->where('PRODUCT_LINE', 'like', "%" .'LENDING' ."%")
        ->limit(100);

        return $data;

    }


    public function getFixedRate($arrangementId, $startDate)
    {
        $timestamp = strtotime($startDate);
        $Year = date("Y", $timestamp);
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.AA_ARR_INTEREST")
        ->select('FIXED_RATE')
       // ->whereYear('BATCH_DATE',$startDate->format('Y'))
        ->where(DB::raw("LEFT(ID, 12)"), $arrangementId)
        ->where(DB::raw("RIGHT(ID, 10)"), 'like', "%" .$Year ."%")
        ->orderBy('ID', 'DESC')
        ->limit(1);

        return $data;
    }

    public function getMaturityDate($arrangementId, $startDate)
    {
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.AA_ACCOUNT_DETAILS")
        ->select('MATURITY_DATE', 'RENEWAL_DATE', 'START_DATE')
        ->where('ID', $arrangementId)
        ->where('BATCH_DATE', $startDate)
        ->limit(1)
        ->orderBy('MATURITY_DATE', 'DESC');

        return $data;
    }

    public function getTermAmount($arrangementId)
    {
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.AA_ARR_TERM_AMOUNT")
        ->select('*')
        ->where(DB::raw("LEFT(ID, 12)"), $arrangementId)
        ->orderBy('ID', 'DESC')
        ->limit(1);

        return $data;
    }

    public function getSingleAccount($acNo, $date)
    {
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.ACCOUNT")
        ->select('WORKING_BALANCE', 'LIMIT_REF')
        ->where('ACCOUNT_NUMBER', $acNo)
        ->where('BATCH_DATE', $date)
        ->limit(1)
        ->orderBy('ACCOUNT_NUMBER', 'DESC');


        return $data;
    }

    public function getLimitCheck($acNo, $date)
    {
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.LIMIT")
        ->select('ACCOUNT', 'EXPIRY_DATE', 'AVAIL_AMT')
        ->where('ACCOUNT', 'like', "%" .$acNo."%")
        ->where('BATCH_DATE', $date)
        ->limit(1)
        ->orderBy('ACCOUNT', 'DESC');

        return $data;
    }

    public function getRateRk($acNo, $date)
    {
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.ACCOUNT_DEBIT_INT")
        ->select('DR_INT_RATE')
        ->where(DB::raw("LEFT(ID, 10)"), $acNo)
        ->where('BATCH_DATE', $date)
        ->limit(1)
        ->orderBy('INSERT_DATE', 'DESC');

        return $data;
    }

    public function getCategory($category)
    {
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.CATEGORY")
        ->select('SHORT_NAME')
        ->where('ID', $category);

        return $data;
    }

    public function getCompany($co_code)
    {
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.COMPANY")
        ->select('COMPANY_NAME')
        ->where('ID', $co_code);

        return $data;
    }

    public function getDateSchecule($arrId)
    {
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.AA_SCHEDULED_ACTIVITY")
        ->select(DB::raw("LEFT(LAST_DATE, 8) AS LAST_DATE, LEFT(NEXT_DATE, 8) AS NEXT_DATE"))
        ->where('ID', $arrId)
        ->limit(1);

        return $data;
    }



     public function getAccountRK($arrCusNo, $date)
    {


        $data = [];
        $data = DB::connection("db2")->table("MIS.COLLECTION")
        ->selectRaw('ACCOUNT_NUMBER,MATURITY_DATE,MAX(BATCH_DATE) AS BATCH_DATE,MAX(OUTSTANDING) AS OUTSTANDING,LOAN_CCY,PRODUCT_LOAN,PRINCIPLE_AMOUNT,CURRENT_INTEREST_RATE,TENOR,FACILITY_TYPE,LIMIT_ID')
        ->where('TANGGAL_DATA', $date)
        ->whereIn(DB::raw("ACCOUNT_NUMBER"), $arrCusNo)
        //->where('LOAN_STATUS', 'CURRENT')
        ->groupBy('ACCOUNT_NUMBER', 'MATURITY_DATE', 'LOAN_CCY', 'PRODUCT_LOAN', 'PRINCIPLE_AMOUNT', 'CURRENT_INTEREST_RATE', 'TENOR','FACILITY_TYPE','LIMIT_ID')
        ->limit(100);
        return $data;
    }

    public function getAccountAnjak($AccNo, $date)
    {
        $timestamp = strtotime($date);
        // Format the timestamp into a date format
        $Year = date("Y", $timestamp);
        $data = [];
        $data = DB::connection("db2")->table("MIS.COLLECTION")
        ->selectRaw('ACCOUNT_NUMBER,MATURITY_DATE,MAX(BATCH_DATE) AS BATCH_DATE,MAX(OUTSTANDING) AS OUTSTANDING,LOAN_CCY,PRODUCT_LOAN,PRINCIPLE_AMOUNT,CURRENT_INTEREST_RATE,TENOR,FACILITY_TYPE,LIMIT_ID')
        ->whereYear('TANGGAL_DATA', $Year)
        ->whereIn('PAYIN_ACCOUNT', $AccNo)
        ->where('LOAN_STATUS', 'CURRENT')
        //->where('LOAN_STATUS', 'CURRENT')
        ->groupBy('ACCOUNT_NUMBER', 'MATURITY_DATE', 'LOAN_CCY', 'PRODUCT_LOAN', 'PRINCIPLE_AMOUNT', 'CURRENT_INTEREST_RATE', 'TENOR','FACILITY_TYPE','LIMIT_ID')
        ->limit(100);
        return $data;

    return $data;
    }

    public function getLimit($cus_id, $periode)
    {
        $timestamp = strtotime($periode);
        // Format the timestamp into a date format
        $Year = date("Y", $timestamp);
        $data = [];
        $data = DB::connection("db2")->table("STG_DB.LIMIT AS LM")
        ->select('*')
        ->join('STG_DB.LIMIT_REFERENCE AS LR', 'LM.LIMIT_PRODUCT', '=', 'LR.ID')
        ->join('STG_DB.COMPANY AS CP', 'LM.CO_CODE', '=', 'CP.ID')
        ->join('STG_DB.CATEGORY AS CT', 'LM.LIMIT_PRODUCT', '=', 'CT.ID')
        ->select('LM.ACCOUNT', 'LM.LIMIT_CURRENCY', 'LM.APPROVAL_DATE', 'LM.EXPIRY_DATE', 'LM.AVAIL_AMT', 'LM.BATCH_DATE', 'LM.EXPIRY_DATE', 'LR.SHORT_NAME', 'LR.LIMIT_PERCENTAGE', 'CP.COMPANY_NAME', 'CT.SHORT_NAME AS ACCOUNT_TYPE')
        ->where(DB::raw("LEFT(LM.@ID, 10)"), $cus_id)
        ->whereYear('LM.BATCH_DATE', $Year)
        ->limit(100);
        return $data;
    }



}